<?php
namespace App\Http\Controllers;


use App\Http\Resources\ClientCategoriesResource;
use App\Http\Resources\ClientsCollection;
use App\Http\Resources\ClientCategoriesCollection;

use App\Http\Resources\FaqsCollection;
use App\Http\Resources\FeaturesCollection;
use App\Http\Resources\FeaturesResource;
use App\Http\Resources\MenusCollection;
use App\Http\Resources\PagesResource;
use App\Http\Resources\ServicesCollection;
use App\Http\Resources\SettingsCollection;
use App\Http\Resources\SlidersCollection;
use App\Http\Resources\SolutionsCollection;
use App\Http\Resources\SolutionsMenuCollection;
use App\Models\ClientCategories;
use App\Models\Clients;
use App\Models\DemoServices;
use App\Models\Faqs;
use App\Models\Features;
use App\Models\ServiceRequests;
use App\Models\Slider;
use App\Models\Page;
use App\Models\Solutions;
use Backpack\PageManager\PageManagerServiceProvider;
use App\Models\Setting;
use Carbon\Carbon;
use GuzzleHttp\Client;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;

class HomeApiController extends Controller
{
    public function menu(Request $request)
    {
        $menus = Page::where('show_menu','1')->orderBy('sort','asc')->get();
        $solutions =  Solutions::with(['solutiondetails'])->where('category','SOLUTION')->active()->orderBy('created_at')->get();
        $mobility_solutions = Solutions::with(['solutiondetails'])->where('category','MOBILITY SOLUTION')->active()->orderBy('created_at')->get();
        return returnApi(['status' => 1, 'message' => '','menus'=>new MenusCollection($menus),
        'solutions'=>new SolutionsMenuCollection($solutions),
        'mobility_solutions'=>new SolutionsMenuCollection($mobility_solutions)]
        );
    }
    public function home(Request $request)
    {
        $seo_title = Setting::where('key','seo_title')->first();
        $seo_desc = Setting::where('key','seo_desc')->first();
        $seo_keywords = Setting::where('key','seo_keywords')->first();
        $sliders = Slider::active()->orderBy('created_at')->get();
        $about = Page::where('slug','core')->first();
        $scale = Page::where('slug','scale')->first();
        $features = Features::active()->orderBy('created_at')->limit(10)->get();
        $clients = Clients::active()->orderBy('created_at')->limit(10)->get();
        return returnApi(['status' => 1, 'message' => '',

                'seo_title'=>@$seo_title->value,
                'seo_description'=>@$seo_desc->value,
                'seo_keywords'=>@$seo_keywords->value,
                'sliders'=>new SlidersCollection($sliders)

            ,'scale_section'=>new PagesResource($scale)
            ,'about_section'=>new PagesResource($about)
            ,'features'=>new FeaturesCollection($features)
            ,'clients'=>new ClientsCollection($clients)

            ]



        );
    }

    public function page(Request $request)
    {
        $about = Page::where('slug',$request->slug)->first();
        return returnApi(['status' => 1, 'message' => ''
                ,'page'=>new PagesResource($about)

            ]



        );
    }
    public function settings(Request $request)
    {
        $settings = Setting::all();
        return returnApi(['status' => 1, 'message' => '','settings'=>new SettingsCollection($settings)


            ]



        );
    }

    public function faqsabout(Request $request)
    {
        $faqs = Faqs::active()->where('show_about','SHOW')->get();
        return returnApi(['status' => 1, 'message' => '','faqs'=>new FaqsCollection($faqs)]);
    }
    public function faqs(Request $request)
    {
        $faqs = Faqs::active()->get();
        return returnApi(['status' => 1, 'message' => '','faqs'=>new FaqsCollection($faqs)]);
    }
    public function clientCategories(Request $request)
    {
        $clients = ClientCategories::with('clients')->active()->orderBy('created_at')->get();
         return returnApi(['status' => 1, 'message' => '',
                'clients'=>new ClientCategoriesCollection($clients)
            ]
        );
    }
    public function clients(Request $request)
    {
        if(isset($request->id)){
        $clients = Clients::active()->where('category_id',$request->id)->orderBy('created_at')->get();
        }
        else 
        {
             $clients = Clients::active()->limit(10)->orderBy('created_at')->get();
        }
         return returnApi(['status' => 1, 'message' => '',
                'clients'=>new ClientsCollection($clients)
            ]
        );
    }
    public function contactUs(Request $request)
    {
        $requestData = $request->all();
        $validator = Validator::make($requestData,[
            'name' => 'required',
             
            'phone' => 'required',
            'message' => 'required'
        ]);

        if ($validator->fails()) {
            return response()->json([
                'errors' => $validator->errors()
            ], 422);
        }
        $requestData = (array_merge($requestData,['status'=>'New']));
         $requestInfo = ServiceRequests::create($requestData);

        return returnApi(['status' => 1, 'message' => 'Message sent']);

    }
    public function requestDemo(Request $request)
    {
        $requestData = $request->all();

        $validator = Validator::make($requestData,[
             
            'name' => 'required',
             
            'phone' => 'required',
             
            'message' => 'required'
        ]);

        if ($validator->fails()) {
            return response()->json([
                'errors' => $validator->errors()
            ], 422);
        }
        $requestData = (array_merge($requestData,['status'=>'New','service'=>json_encode($requestData['service'])]));
        $requestInfo = ServiceRequests::create($requestData);
        $services = [] ;
        $storeSerivces = [] ;

        foreach ($request->service as $service)
        {
            $re = new ServiceRequests();
            $storeSerivces[] = $re->services[$service];
            $services[] = [
                'item'=>[
                "itemCode"=> $service,
                "item"=>$service],
                'quantity'=>['quantity'=>[
            'primeQty'=>[
                "value"=> 1,"uom"=> "SRV"
                ]
                ]
        ]
            ];

//            $services[] = '{"item": {},"quantity": {"quantity": {"primeQty": {"value": 1,"uom": "SRV"}}}} ';
        }
//         $services = implode(',',$services);
        $f1['SalesQuotationRequest'][]=[
            "book"=> "SalesQuotationRequest$#SQR",
            "code"=>"SQR0000001@draft",
            "term"=> "SalesQuotationRequest$#SQR",
            "valueDate"=> Carbon::today()->format('d-m-Y'),
            "description1"=> $requestInfo->company_name,
            "description2"=> $requestInfo->name,
            "description3"=> $requestInfo->num_users,
            "description4"=> $requestInfo->phone,
            "description5"=> $requestInfo->email,
            "money"=>["currency"=> "KWD"],
            "legalEntity"=> "001","sector"=>"PUBLIC","branch"=> "PUBLIC","department"=> "PUBLIC","analysisSet"=> "PUBLIC",
            "details"=>$services
        ];
           $curl = curl_init();

        curl_setopt_array($curl, array(
            CURLOPT_URL => 'https://crm.exceed-erp.com/erp/integrator?integrator=SQR&mode=write&user=import&pass=import%40exceed',
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'POST',
            CURLOPT_POSTFIELDS =>json_encode($f1),
            CURLOPT_HTTPHEADER => array(
                'Content-Type: application/json'
            ),
        ));

        $response = curl_exec($curl);

        curl_close($curl);
         $requestInfo->service = json_encode($storeSerivces);
         $requestInfo->api_response = json_encode($response);
        $requestInfo->save();
        return returnApi(['status' => 1, 'message' => 'Message sent']);

    }

    public function requestServices(Request $request)
    {
        $services = DemoServices::active()->get();
        return returnApi(['status' => 1, 'message' => '','services'=>new ServicesCollection($services)]);
    }
}
