<?php

namespace App;

use Backpack\CRUD\app\Library\CrudPanel\CrudPanelFacade as CRUD;

trait PageTemplates
{
    /*
    |--------------------------------------------------------------------------
    | Page Templates for Backpack\PageManager
    |--------------------------------------------------------------------------
    |
    | Each page template has its own method, that define what fields should show up using the Backpack\CRUD API.
    | Use snake_case for naming and PageManager will make sure it looks pretty in the create/update form
    | template dropdown.
    |
    | Any fields defined here will show up after the standard page fields:
    | - select template
    | - page name (only seen by admins)
    | - page title
    | - page slug
    */

    private function services()
    {
        $this->crud->addField([   // CustomHTML
            'name' => 'metas_separator',
            'type' => 'custom_html',
            'value' => '<br><h2>'.trans('backpack::pagemanager.metas').'</h2><hr>',
        ]);
        $this->crud->addField([
            'name' => 'meta_title',
            'label' => trans('backpack::pagemanager.meta_title'),
            'fake' => true,
            'store_in' => 'extras',
        ]);
        $this->crud->addField([
            'name' => 'meta_description',
            'label' => trans('backpack::pagemanager.meta_description'),
            'fake' => true,
            'store_in' => 'extras',
        ]);
        $this->crud->addField([
            'name' => 'meta_keywords',
            'type' => 'textarea',
            'label' => trans('backpack::pagemanager.meta_keywords'),
            'fake' => true,
            'store_in' => 'extras',
        ]);
        $this->crud->addField([   // CustomHTML
            'name' => 'content_separator',
            'type' => 'custom_html',
            'value' => '<br><h2>'.trans('backpack::pagemanager.content').'</h2><hr>',
        ]);
        $this->crud->addField([
            'name' => 'content',
            'label' => trans('backpack::pagemanager.content'),
            'type' => 'summernote',
            'placeholder' => trans('backpack::pagemanager.content_placeholder'),
        ]);
        $this->crud->addField([
            'name' => 'show_menu',
            'label' => 'show menu',
            'type' => 'select_from_array',
            'options' => [
                '1' => 'Show',
                '0' => 'Hide',
            ],
        ]);
        $this->crud->addField([
            'name' => 'sort',
            'label' => 'Sort',
            'type' => 'number',
        ]);
        CRUD::field('image')->type('upload')->withFiles([
            'fileNamer' => \Backpack\CRUD\app\Library\Uploaders\Support\FileNameGenerator::class,
        ])

// alternativelly you can pass a closure:
            ->withFiles([
                'fileNamer' => function($file, $uploader) { return 'the_file_name.png'; },
            ]);
    }

    private function about_us()
    {
        $this->crud->addField([
            'name' => 'content',
            'label' => trans('backpack::pagemanager.content'),
            'type' => 'summernote',
            'placeholder' => trans('backpack::pagemanager.content_placeholder'),
        ]);
        $this->crud->addField([
            'name' => 'show_menu',
            'label' => 'show menu',
            'type' => 'select_from_array',
            'options' => [
                '1' => 'Show',
                '0' => 'Hide',
            ],
        ]);
        $this->crud->addField([
            'name' => 'sort',
            'label' => 'Sort',
            'type' => 'number',
        ]);
        CRUD::field('image')->type('upload')->withFiles([
            'fileNamer' => \Backpack\CRUD\app\Library\Uploaders\Support\FileNameGenerator::class,
        ])

// alternativelly you can pass a closure:
            ->withFiles([
                'fileNamer' => function($file, $uploader) { return 'the_file_name.png'; },
            ]);
    }
}
